package com.nokia.tsl_data.service;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.nokia.tsl_data.dao.TslDao;
import com.nokia.tsl_data.service.exception.TslTaskException;

import lombok.extern.slf4j.Slf4j;

/**
 * 任务
 */
@Slf4j
@Service
public class TslTaskService {

    private final TslReportService tslReportService;
    private final TslWaraHouseService tslWaraHouseService;
    private final TslDao tslDao;

    @Value("${tslTask.outputPath}")
    private String outputPath;

    private String outputFileName = "投诉清单各地市投诉率";

    public TslTaskService(TslReportService tslReportService, TslWaraHouseService tslWaraHouseService, TslDao tslDao) {
        this.tslReportService = tslReportService;
        this.tslWaraHouseService = tslWaraHouseService;
        this.tslDao = tslDao;
    }

    /**
     * 定时任务
     * 
     * @return
     */
    public Runnable cronTask() {
        return new Runnable() {

            @Override
            public void run() {
                // 获取入参 day
                String day = new SimpleDateFormat("yyyyMMdd")
                        .format(new Date(System.currentTimeMillis() - 1000L * 3600 * 24));

                try {
                    dataWaraHouseTask(day);
                    reportGenerateTask(day);
                } catch (Exception e) {
                    log.error("定时任务出错--{}", e.getMessage());
                    e.printStackTrace();
                }
            }

        };
    }

    /**
     * 报表生成任务
     * 
     * @param day
     * @throws TslTaskException
     */
    public void reportGenerateTask(String day) throws TslTaskException {
        File file = new File(outputPath + day);
        if (!file.exists()) {
            file.mkdirs();
        }
        String dayId = day.substring(0, 4) + "-" + day.substring(4, 6) + "-" + day.substring(6);
        int qualityCountForDay = tslDao.selectQualityCountForDay(dayId);
        if (qualityCountForDay == 0) {
            throw new TslTaskException("he_d_high_quality表缺少数据");
        }
        int compCountForDay = tslDao.selectCompCountForDay(day);
        if (compCountForDay == 0) {
            throw new TslTaskException("he_d_mobile_comp表缺少数据");
        }
        tslReportService.workbookToFile(day, file.getAbsolutePath() + "/" + outputFileName + day + ".xlsx");
    }

    /**
     * 数据入库任务
     * 
     * @param day
     * @throws IOException
     * @throws FileNotFoundException
     */
    @Transactional(rollbackFor = Exception.class)
    public void dataWaraHouseTask(String day) throws FileNotFoundException, IOException {
        tslWaraHouseService.waraHouseHighQuality(day);
        tslWaraHouseService.waraHouseMobileComp(day);
    }
}
